"""
:mod:`zsl.interface.webservice.importer`
----------------------------------------
"""
from __future__ import unicode_literals
import sys
import os

# TODO: Consider removing automatic path initialization!
sys.path.append(os.path.join(os.path.dirname(__file__), '..', '..', '..'))
